@@ -207,16 +207,17 @@ sub on_presets_changed {
     $self->{on_presets_changed} = $cb;
 }
 
-# This method is supposed to be called whenever new values are loaded
-# or changed by user (so also when a preset is loaded).
-# propagate event to the parent
+# This method is called whenever an option field is changed by the user.
+# Propagate event to the parent through the 'on_value_changed' callback
+# and call _update.
 sub _on_value_change {
-    my $self = shift;
-    
-    $self->{on_value_change}->(@_) if $self->{on_value_change};
-    $self->_update;
+    my ($self, $key, $value) = @_;
+    $self->{on_value_change}->($key, $value) if $self->{on_value_change};
+    $self->_update({ $key => 1 });
 }
 
+# Override this to capture changes of configuration caused either by loading or switching a preset,
+# or by a user changing an option field.
 sub _update {}
 
 sub _on_presets_changed {
@@ -298,15 +299,20 @@ sub on_select_preset {
     my $preset_config = $self->get_preset_config($preset);
     eval {
         local $SIG{__WARN__} = Slic3r::GUI::warning_catcher($self);
+        my %keys_modified = ();
         foreach my $opt_key (@{$self->{config}->get_keys}) {
-            $self->{config}->set($opt_key, $preset_config->get($opt_key))
-                if $preset_config->has($opt_key);
+            if ($preset_config->has($opt_key)) {
+                if ($self->{config}->serialize($opt_key) ne $preset_config->serialize($opt_key)) {
+                    $self->{config}->set($opt_key, $preset_config->get($opt_key));
+                    $keys_modified{$opt_key} = 1;
+                }
+            }
         }
         ($preset->default || $preset->external)
             ? $self->{btn_delete_preset}->Disable
             : $self->{btn_delete_preset}->Enable;
         
-        $self->_update;
+        $self->_update(\%keys_modified);
         $self->on_preset_loaded;
         $self->reload_config;
         $Slic3r::GUI::Settings->{presets}{$self->name} = $preset->file ? basename($preset->file) : '';
@@ -384,13 +390,17 @@ sub update_tree {
 sub update_dirty {
     my $self = shift;
     
+    my $list_updated;
     foreach my $i ($self->{default_suppressed}..$#{$self->{presets}}) {
         my $preset = $self->get_preset($i);
-        $self->{presets_choice}->SetString(
-            $i - $self->{default_suppressed}, 
-            ($i == $self->current_preset && $self->is_dirty) ? $preset->name . " (modified)" : $preset->name);
+        my $label  = ($i == $self->current_preset && $self->is_dirty) ? $preset->name . " (modified)" : $preset->name;
+        my $idx    = $i - $self->{default_suppressed};
+        if ($self->{presets_choice}->GetString($idx) ne $label) {
+            $self->{presets_choice}->SetString($idx, $label);
+            $list_updated = 1;
+        }
     }
-    $self->{presets_choice}->SetSelection($self->current_preset - $self->{default_suppressed});  # http://trac.wxwidgets.org/ticket/13769
+    $self->{presets_choice}->SetSelection($self->current_preset - $self->{default_suppressed}) if ($list_updated);  # http://trac.wxwidgets.org/ticket/13769
     $self->_on_presets_changed;
 }
 
@@ -463,12 +473,17 @@ sub load_config {
     my $self = shift;
     my ($config) = @_;
     
+    my %keys_modified = ();
     foreach my $opt_key (@{$self->{config}->diff($config)}) {
         $self->{config}->set($opt_key, $config->get($opt_key));
+        $keys_modified{$opt_key} = 1;
+    }
+    if (keys(%keys_modified)) {
         $self->update_dirty;
+        # Initialize UI components with the config values.
+        $self->reload_config;
+        $self->_update(\%keys_modified);
     }
-    $self->reload_config;
-    $self->_update;
 }
 
 sub get_preset_config {
@@ -523,6 +538,7 @@ sub build {
         max_volumetric_extrusion_rate_slope_positive max_volumetric_extrusion_rate_slope_negative
         perimeter_speed small_perimeter_speed external_perimeter_speed infill_speed 
         solid_infill_speed top_solid_infill_speed support_material_speed 
+        support_material_xy_spacing
         support_material_interface_speed bridge_speed gap_fill_speed
         travel_speed
         first_layer_speed
@@ -532,8 +548,8 @@ sub build {
         brim_width
         support_material support_material_threshold support_material_enforce_layers
         raft_layers
-        support_material_pattern support_material_with_sheath support_material_spacing support_material_angle
-        support_material_interface_layers support_material_interface_spacing
+        support_material_pattern support_material_with_sheath support_material_spacing support_material_synchronize_layers support_material_angle
+        support_material_interface_layers support_material_interface_spacing support_material_interface_contact_loops
         support_material_contact_distance support_material_buildplate_only dont_support_bridges
         notes
         complete_objects extruder_clearance_radius extruder_clearance_height
@@ -547,7 +563,8 @@ sub build {
         external_perimeter_extrusion_width infill_extrusion_width solid_infill_extrusion_width 
         top_infill_extrusion_width support_material_extrusion_width
         infill_overlap bridge_flow_ratio
-        xy_size_compensation threads resolution
+        clip_multipart_objects xy_size_compensation threads resolution
+        wipe_tower wipe_tower_x wipe_tower_y wipe_tower_width wipe_tower_per_color_wipe
     ));
     $self->{config}->set('print_settings_id', '');
     
@@ -636,6 +653,7 @@ sub build {
         {
             my $optgroup = $page->new_optgroup('Raft');
             $optgroup->append_single_option_line('raft_layers');
+#            $optgroup->append_single_option_line('raft_contact_distance');
         }
         {
             my $optgroup = $page->new_optgroup('Options for support material and raft');
@@ -646,8 +664,11 @@ sub build {
             $optgroup->append_single_option_line('support_material_angle');
             $optgroup->append_single_option_line('support_material_interface_layers');
             $optgroup->append_single_option_line('support_material_interface_spacing');
+            $optgroup->append_single_option_line('support_material_interface_contact_loops');
             $optgroup->append_single_option_line('support_material_buildplate_only');
+            $optgroup->append_single_option_line('support_material_xy_spacing');
             $optgroup->append_single_option_line('dont_support_bridges');
+            $optgroup->append_single_option_line('support_material_synchronize_layers');
         }
     }
     
@@ -706,6 +727,14 @@ sub build {
             $optgroup->append_single_option_line('ooze_prevention');
             $optgroup->append_single_option_line('standby_temperature_delta');
         }
+        {
+            my $optgroup = $page->new_optgroup('Wipe tower');
+            $optgroup->append_single_option_line('wipe_tower');
+            $optgroup->append_single_option_line('wipe_tower_x');
+            $optgroup->append_single_option_line('wipe_tower_y');
+            $optgroup->append_single_option_line('wipe_tower_width');
+            $optgroup->append_single_option_line('wipe_tower_per_color_wipe');
+        }
         {
             my $optgroup = $page->new_optgroup('Advanced');
             $optgroup->append_single_option_line('interface_shells');
@@ -737,6 +766,7 @@ sub build {
         }
         {
             my $optgroup = $page->new_optgroup('Other');
+            $optgroup->append_single_option_line('clip_multipart_objects');
             $optgroup->append_single_option_line('xy_size_compensation');
             $optgroup->append_single_option_line('threads') if $Slic3r::have_threads;
             $optgroup->append_single_option_line('resolution');
@@ -793,9 +823,12 @@ sub build {
     }
 }
 
+# Slic3r::GUI::Tab::Print::_update is called after a configuration preset is loaded or switched, or when a single option is modifed by the user.
 sub _update {
-    my ($self) = @_;
-    
+    # $keys_modified is a reference to hash with modified keys set to 1, unmodified keys missing.
+    my ($self, $keys_modified) = @_;
+    $keys_modified //= {};
+
     my $config = $self->{config};
     
     if ($config->spiral_vase && !($config->perimeters == 1 && $config->top_solid_layers == 0 && $config->fill_density == 0)) {
@@ -805,6 +838,7 @@ sub _update {
             . "- no top solid layers\n"
             . "- 0% fill density\n"
             . "- no support material\n"
+            . "- no ensure_vertical_shell_thickness\n"
             . "\nShall I adjust those settings in order to enable Spiral Vase?",
             'Spiral Vase', wxICON_WARNING | wxYES | wxNO);
         if ($dialog->ShowModal() == wxID_YES) {
@@ -813,6 +847,7 @@ sub _update {
             $new_conf->set("top_solid_layers", 0);
             $new_conf->set("fill_density", 0);
             $new_conf->set("support_material", 0);
+            $new_conf->set("ensure_vertical_shell_thickness", 0);
             $self->load_config($new_conf);
         } else {
             my $new_conf = Slic3r::Config->new;
@@ -821,6 +856,10 @@ sub _update {
         }
     }
 
+    if ($keys_modified->{'layer_height'}) {
+        # If the user had set the variable layer height, reset it and let the user know.
+    }
+
     if ($config->support_material) {
         # Ask only once.
         if (! $self->{support_material_overhangs_queried}) {
@@ -906,17 +945,20 @@ sub _update {
     # perimeter_extruder uses the same logic as in Print::extruders()
     $self->get_field('perimeter_extruder')->toggle($have_perimeters || $have_brim);
     
-    my $have_support_material = $config->support_material || $config->raft_layers > 0;
+    my $have_raft = $config->raft_layers > 0;
+    my $have_support_material = $config->support_material || $have_raft;
     my $have_support_interface = $config->support_material_interface_layers > 0;
+    my $have_support_soluble = $have_support_material && $config->support_material_contact_distance == 0;
     $self->get_field($_)->toggle($have_support_material)
         for qw(support_material_threshold support_material_pattern support_material_with_sheath
             support_material_spacing support_material_angle
             support_material_interface_layers dont_support_bridges
-            support_material_extrusion_width support_material_contact_distance);
+            support_material_extrusion_width support_material_contact_distance support_material_xy_spacing);
     $self->get_field($_)->toggle($have_support_material && $have_support_interface)
         for qw(support_material_interface_spacing support_material_interface_extruder
-            support_material_interface_speed);
-    
+            support_material_interface_speed support_material_interface_contact_loops);
+    $self->get_field('support_material_synchronize_layers')->toggle($have_support_soluble);
+
     $self->get_field('perimeter_extrusion_width')->toggle($have_perimeters || $have_skirt || $have_brim);
     $self->get_field('support_material_extruder')->toggle($have_support_material || $have_skirt);
     $self->get_field('support_material_speed')->toggle($have_support_material || $have_brim || $have_skirt);
@@ -928,6 +970,10 @@ sub _update {
     my $have_ooze_prevention = $config->ooze_prevention;
     $self->get_field($_)->toggle($have_ooze_prevention)
         for qw(standby_temperature_delta);
+
+    my $have_wipe_tower = $config->wipe_tower;
+    $self->get_field($_)->toggle($have_wipe_tower)
+        for qw(wipe_tower_x wipe_tower_y wipe_tower_width wipe_tower_per_color_wipe);
 }
 
 sub hidden_options { !$Slic3r::have_threads ? qw(threads) : () }
@@ -942,7 +988,7 @@ sub build {
     my $self = shift;
     
     $self->init_config_options(qw(
-        filament_colour filament_diameter filament_notes filament_max_volumetric_speed extrusion_multiplier
+        filament_colour filament_diameter filament_type filament_soluble filament_notes filament_max_volumetric_speed extrusion_multiplier filament_density filament_cost
         temperature first_layer_temperature bed_temperature first_layer_bed_temperature
         fan_always_on cooling
         min_fan_speed max_fan_speed bridge_fan_speed disable_fan_first_layers
@@ -957,6 +1003,8 @@ sub build {
             $optgroup->append_single_option_line('filament_colour', 0);
             $optgroup->append_single_option_line('filament_diameter', 0);
             $optgroup->append_single_option_line('extrusion_multiplier', 0);
+            $optgroup->append_single_option_line('filament_density', 0);
+            $optgroup->append_single_option_line('filament_cost', 0);
         }
     
         {
@@ -1027,7 +1075,11 @@ sub build {
     {
         my $page = $self->add_options_page('Advanced', 'wrench.png');
         {
-            my $optgroup = $page->new_optgroup('Print speed override');
+            my $optgroup = $page->new_optgroup('Filament properties');
+            $optgroup->append_single_option_line('filament_type', 0);
+            $optgroup->append_single_option_line('filament_soluble', 0);
+            
+            $optgroup = $page->new_optgroup('Print speed override');
             $optgroup->append_single_option_line('filament_max_volumetric_speed', 0);
         }
     }
@@ -1046,8 +1098,10 @@ sub build {
     }
 }
 
+# Slic3r::GUI::Tab::Filament::_update is called after a configuration preset is loaded or switched, or when a single option is modifed by the user.
 sub _update {
-    my ($self) = @_;
+    # $keys_modified is a reference to hash with modified keys set to 1, unmodified keys missing.
+    my ($self, $keys_modified) = @_;
     
     $self->_update_description;
     
@@ -1104,12 +1158,13 @@ sub build {
         gcode_flavor use_relative_e_distances
         serial_port serial_speed
         octoprint_host octoprint_apikey
-        use_firmware_retraction pressure_advance
-        use_volumetric_e
-        start_gcode end_gcode before_layer_gcode layer_gcode toolchange_gcode
+        use_firmware_retraction
+        use_volumetric_e set_and_wait_temperatures variable_layer_height
+        single_extruder_multi_material start_gcode end_gcode before_layer_gcode layer_gcode toolchange_gcode
         nozzle_diameter extruder_offset
-        retract_length retract_lift retract_speed retract_restart_extra retract_before_travel retract_layer_change wipe
+        retract_length retract_lift retract_speed deretract_speed retract_before_wipe retract_restart_extra retract_before_travel retract_layer_change wipe
         retract_length_toolchange retract_restart_extra_toolchange
+        printer_notes
     ));
     $self->{config}->set('printer_settings_id', '');
     
@@ -1166,6 +1221,7 @@ sub build {
                     min         => 1,
                 );
                 $optgroup->append_single_option_line($option);
+                $optgroup->append_single_option_line('single_extruder_multi_material');
             }
             $optgroup->on_change(sub {
                 my ($opt_id) = @_;
@@ -1309,7 +1365,8 @@ sub build {
             $optgroup->append_single_option_line('use_relative_e_distances');
             $optgroup->append_single_option_line('use_firmware_retraction');
             $optgroup->append_single_option_line('use_volumetric_e');
-            $optgroup->append_single_option_line('pressure_advance');
+            $optgroup->append_single_option_line('variable_layer_height');
+            $optgroup->append_single_option_line('set_and_wait_temperatures');
         }
     }
     {
@@ -1361,8 +1418,22 @@ sub build {
         }
     }
     
+    {
+        my $page = $self->add_options_page('Notes', 'note.png');
+        {
+            my $optgroup = $page->new_optgroup('Notes',
+                label_width => 0,
+            );
+            my $option = $optgroup->get_option('printer_notes');
+            $option->full_width(1);
+            $option->height(250);
+            $optgroup->append_single_option_line($option);
+        }
+    }
+
     $self->{extruder_pages} = [];
     $self->_build_extruder_pages;
+
     $self->_update_serial_ports if (!$params{no_controller});
 }
 
@@ -1378,11 +1449,13 @@ sub _extruders_count_changed {
     $self->{extruders_count} = $extruders_count;
     $self->_build_extruder_pages;
     $self->_on_value_change('extruders_count', $extruders_count);
-    $self->_update;
 }
 
-sub _extruder_options { qw(nozzle_diameter extruder_offset retract_length retract_lift retract_lift_above retract_lift_below retract_speed retract_restart_extra retract_before_travel wipe
-    retract_layer_change retract_length_toolchange retract_restart_extra_toolchange) }
+sub _extruder_options { 
+    qw(nozzle_diameter min_layer_height max_layer_height extruder_offset 
+       retract_length retract_lift retract_lift_above retract_lift_below retract_speed deretract_speed 
+       retract_before_wipe retract_restart_extra retract_before_travel wipe
+       retract_layer_change retract_length_toolchange retract_restart_extra_toolchange) }
 
 sub _build_extruder_pages {
     my $self = shift;
@@ -1410,6 +1483,11 @@ sub _build_extruder_pages {
             my $optgroup = $page->new_optgroup('Size');
             $optgroup->append_single_option_line('nozzle_diameter', $extruder_idx);
         }
+        {
+            my $optgroup = $page->new_optgroup('Layer height limits');
+            $optgroup->append_single_option_line($_, $extruder_idx)
+                for qw(min_layer_height max_layer_height);
+        }
         {
             my $optgroup = $page->new_optgroup('Position (for multi-extruder printers)');
             $optgroup->append_single_option_line('extruder_offset', $extruder_idx);
@@ -1429,7 +1507,7 @@ sub _build_extruder_pages {
             }
             
             $optgroup->append_single_option_line($_, $extruder_idx)
-                for qw(retract_speed retract_restart_extra retract_before_travel retract_layer_change wipe);
+                for qw(retract_speed deretract_speed retract_restart_extra retract_before_travel retract_layer_change wipe retract_before_wipe);
         }
         {
             my $optgroup = $page->new_optgroup('Retraction when tool is disabled (advanced settings for multi-extruder setups)');
@@ -1453,15 +1531,20 @@ sub _build_extruder_pages {
     }
     
     # rebuild page list
+    my @pages_without_extruders = (grep $_->{title} !~ /^Extruder \d+/, @{$self->{pages}});
+    my $page_notes = pop @pages_without_extruders;
     @{$self->{pages}} = (
-        (grep $_->{title} !~ /^Extruder \d+/, @{$self->{pages}}),
+        @pages_without_extruders,
         @{$self->{extruder_pages}}[ 0 .. $self->{extruders_count}-1 ],
+        $page_notes
     );
     $self->update_tree;
 }
 
+# Slic3r::GUI::Tab::Printer::_update is called after a configuration preset is loaded or switched, or when a single option is modifed by the user.
 sub _update {
-    my ($self) = @_;
+    # $keys_modified is a reference to hash with modified keys set to 1, unmodified keys missing.
+    my ($self, $keys_modified) = @_;
     
     my $config = $self->{config};
     
@@ -1483,6 +1566,7 @@ sub _update {
     
     my $have_multiple_extruders = $self->{extruders_count} > 1;
     $self->get_field('toolchange_gcode')->toggle($have_multiple_extruders);
+    $self->get_field('single_extruder_multi_material')->toggle($have_multiple_extruders);
     
     for my $i (0 .. ($self->{extruders_count}-1)) {
         my $have_retract_length = $config->get_at('retract_length', $i) > 0;
@@ -1505,8 +1589,12 @@ sub _update {
         
         # some options only apply when not using firmware retraction
         $self->get_field($_, $i)->toggle($retraction && !$config->use_firmware_retraction)
-            for qw(retract_speed retract_restart_extra wipe);
-        if ($config->use_firmware_retraction && $config->get_at('wipe', $i)) {
+            for qw(retract_speed deretract_speed retract_before_wipe retract_restart_extra wipe);
+
+        my $wipe = $config->get_at('wipe', $i);
+        $self->get_field('retract_before_wipe', $i)->toggle($wipe);
+
+        if ($config->use_firmware_retraction && $wipe) {
             my $dialog = Wx::MessageDialog->new($self,
                 "The Wipe option is not available when using the Firmware Retraction mode.\n"
                 . "\nShall I disable it in order to enable Firmware Retraction?",
