@@ -25,12 +25,12 @@ my %opt = ();
 }
 
 {
-    my $model = Slic3r::Format::AMF->read_file($ARGV[0]);
+    my $model = Slic3r::Model->load_amf(Slic3r::encode_path($ARGV[0]));
     my $output_file = $ARGV[0];
     $output_file =~ s/\.amf(?:\.xml)?$/\.stl/i;
     
     printf "Writing to %s\n", basename($output_file);
-    Slic3r::Format::STL->write_file($output_file, $model, binary => !$opt{ascii});
+    $model->store_stl(Slic3r::encode_path($output_file), binary => !$opt{ascii});
 }
 
 
