@@ -10,24 +10,10 @@
 #include <string>
 #include <map>
 
+#include <boost/log/trivial.hpp>
+
 namespace Slic3r {
 
-LayerRegion::LayerRegion(Layer *layer, PrintRegion *region)
-:   _layer(layer),
-    _region(region)
-{
-}
-
-LayerRegion::~LayerRegion()
-{
-}
-
-Layer*
-LayerRegion::layer()
-{
-    return this->_layer;
-}
-
 Flow
 LayerRegion::flow(FlowRole role, bool bridge, double width) const
 {
@@ -50,12 +36,18 @@ void LayerRegion::slices_to_fill_surfaces_clipped()
     // that combine_infill() turns some fill_surface into VOID surfaces.
 //    Polygons fill_boundaries = to_polygons(STDMOVE(this->fill_surfaces));
     Polygons fill_boundaries = to_polygons(this->fill_expolygons);
+    // Collect polygons per surface type.
+    std::vector<Polygons> polygons_by_surface;
+    polygons_by_surface.assign(size_t(stCount), Polygons());
+    for (Surface &surface : this->slices.surfaces)
+        polygons_append(polygons_by_surface[(size_t)surface.surface_type], surface.expolygon);
+    // Trim surfaces by the fill_boundaries.
     this->fill_surfaces.surfaces.clear();
-    for (Surfaces::const_iterator surface = this->slices.surfaces.begin(); surface != this->slices.surfaces.end(); ++ surface)
-        surfaces_append(
-            this->fill_surfaces.surfaces,
-            intersection_ex(to_polygons(surface->expolygon), fill_boundaries),
-            surface->surface_type);
+    for (size_t surface_type = 0; surface_type < size_t(stCount); ++ surface_type) {
+        const Polygons &polygons = polygons_by_surface[surface_type];
+        if (! polygons.empty())
+            this->fill_surfaces.append(intersection_ex(polygons, fill_boundaries), SurfaceType(surface_type));
+    }
 }
 
 void
@@ -91,9 +83,9 @@ LayerRegion::make_perimeters(const SurfaceCollection &slices, SurfaceCollection*
     g.process();
 }
 
-//#define EXTERNAL_SURFACES_OFFSET_PARAMETERS CLIPPER_OFFSET_SCALE, ClipperLib::jtMiter, 3.
-//#define EXTERNAL_SURFACES_OFFSET_PARAMETERS CLIPPER_OFFSET_SCALE, ClipperLib::jtMiter, 1.5
-#define EXTERNAL_SURFACES_OFFSET_PARAMETERS CLIPPER_OFFSET_SCALE, ClipperLib::jtSquare, 0.
+//#define EXTERNAL_SURFACES_OFFSET_PARAMETERS ClipperLib::jtMiter, 3.
+//#define EXTERNAL_SURFACES_OFFSET_PARAMETERS ClipperLib::jtMiter, 1.5
+#define EXTERNAL_SURFACES_OFFSET_PARAMETERS ClipperLib::jtSquare, 0.
 
 void
 LayerRegion::process_external_surfaces(const Layer* lower_layer)
@@ -126,26 +118,26 @@ LayerRegion::process_external_surfaces(const Layer* lower_layer)
         // bottom_polygons are used to trim inflated top surfaces.
         fill_boundaries.reserve(number_polygons(surfaces));
         bool has_infill = this->region()->config.fill_density.value > 0.;
-        for (Surfaces::iterator surface = this->fill_surfaces.surfaces.begin(); surface != this->fill_surfaces.surfaces.end(); ++surface) {
-            if (surface->surface_type == stTop) {
+        for (const Surface &surface : this->fill_surfaces.surfaces) {
+            if (surface.surface_type == stTop) {
                 // Collect the top surfaces, inflate them and trim them by the bottom surfaces.
                 // This gives the priority to bottom surfaces.
-                surfaces_append(top, offset_ex(surface->expolygon, float(margin), EXTERNAL_SURFACES_OFFSET_PARAMETERS), *surface);
-            } else if (surface->surface_type == stBottom || (surface->surface_type == stBottomBridge && lower_layer == NULL)) {
+                surfaces_append(top, offset_ex(surface.expolygon, float(margin), EXTERNAL_SURFACES_OFFSET_PARAMETERS), surface);
+            } else if (surface.surface_type == stBottom || (surface.surface_type == stBottomBridge && lower_layer == NULL)) {
                 // Grown by 3mm.
-                surfaces_append(bottom, offset_ex(surface->expolygon, float(margin), EXTERNAL_SURFACES_OFFSET_PARAMETERS), *surface);
-            } else if (surface->surface_type == stBottomBridge) {
-                if (! surface->empty())
-                    bridges.push_back(*surface);
+                surfaces_append(bottom, offset_ex(surface.expolygon, float(margin), EXTERNAL_SURFACES_OFFSET_PARAMETERS), surface);
+            } else if (surface.surface_type == stBottomBridge) {
+                if (! surface.empty())
+                    bridges.push_back(surface);
             }
-            bool internal_surface = surface->surface_type != stTop && ! surface->is_bottom();
-            if (has_infill || surface->surface_type != stInternal) {
+            bool internal_surface = surface.surface_type != stTop && ! surface.is_bottom();
+            if (has_infill || surface.surface_type != stInternal) {
                 if (internal_surface)
                     // Make a copy as the following line uses the move semantics.
-                    internal.push_back(*surface);
-                polygons_append(fill_boundaries, STDMOVE(surface->expolygon));
+                    internal.push_back(surface);
+                polygons_append(fill_boundaries, STDMOVE(surface.expolygon));
             } else if (internal_surface)
-                internal.push_back(STDMOVE(*surface));
+                internal.push_back(STDMOVE(surface));
         }
     }
 
@@ -194,7 +186,7 @@ LayerRegion::process_external_surfaces(const Layer* lower_layer)
                         break;
                     }
                 // Grown by 3mm.
-                Polygons polys = offset(bridges[i].expolygon, float(margin), EXTERNAL_SURFACES_OFFSET_PARAMETERS);
+                Polygons polys = offset(to_polygons(bridges[i].expolygon), float(margin), EXTERNAL_SURFACES_OFFSET_PARAMETERS);
                 if (idx_island == -1) {
                     printf("Bridge did not fall into the source region!\r\n");
                 } else {
@@ -234,6 +226,7 @@ LayerRegion::process_external_surfaces(const Layer* lower_layer)
 
         // 3) Merge the groups with the same group id, detect bridges.
         {
+			BOOST_LOG_TRIVIAL(trace) << "Processing external surface, detecting bridges. layer" << this->layer()->print_z << ", bridge groups: " << n_groups;
             for (size_t group_id = 0; group_id < n_groups; ++ group_id) {
                 size_t n_bridges_merged = 0;
                 size_t idx_last = (size_t)-1;
@@ -262,9 +255,7 @@ LayerRegion::process_external_surfaces(const Layer* lower_layer)
                 BridgeDetector bd(
                     initial,
                     lower_layer->slices,
-                    //FIXME parameters are not correct!
-                    // flow(FlowRole role, bool bridge = false, double width = -1) const;
-                    this->flow(frInfill, true, this->layer()->height).scaled_width()
+                    this->flow(frInfill, true).scaled_width()
                 );
                 #ifdef SLIC3R_DEBUG
                 printf("Processing bridge at layer " PRINTF_ZU ":\n", this->layer()->id());
@@ -281,7 +272,8 @@ LayerRegion::process_external_surfaces(const Layer* lower_layer)
             }
 
             fill_boundaries = STDMOVE(to_polygons(fill_boundaries_ex));
-        }
+			BOOST_LOG_TRIVIAL(trace) << "Processing external surface, detecting bridges - done";
+		}
 
     #if 0
         {
@@ -305,8 +297,10 @@ LayerRegion::process_external_surfaces(const Layer* lower_layer)
             polygons_append(polys, STDMOVE(s1));
             for (size_t j = i + 1; j < top.size(); ++ j) {
                 Surface &s2 = top[j];
-                if (! s2.empty() && surfaces_could_merge(s1, s2))
+                if (! s2.empty() && surfaces_could_merge(s1, s2)) {
                     polygons_append(polys, STDMOVE(s2));
+                    s2.clear();
+                }
             }
             if (s1.surface_type == stTop)
                 // Trim the top surfaces by the bottom surfaces. This gives the priority to the bottom surfaces.
@@ -329,8 +323,10 @@ LayerRegion::process_external_surfaces(const Layer* lower_layer)
         polygons_append(polys, STDMOVE(s1));
         for (size_t j = i + 1; j < internal.size(); ++ j) {
             Surface &s2 = internal[j];
-            if (! s2.empty() && surfaces_could_merge(s1, s2))
+            if (! s2.empty() && surfaces_could_merge(s1, s2)) {
                 polygons_append(polys, STDMOVE(s2));
+                s2.clear();
+            }
         }
         ExPolygons new_expolys = diff_ex(polys, new_polygons);
         polygons_append(new_polygons, to_polygons(new_expolys));
