@@ -54,8 +54,7 @@ PerimeterGenerator::process()
     for (Surfaces::const_iterator surface = this->slices->surfaces.begin();
         surface != this->slices->surfaces.end(); ++surface) {
         // detect how many perimeters must be generated for this island
-        signed short loop_number = this->config->perimeters + surface->extra_perimeters;
-        loop_number--;  // 0-indexed loops
+        const int loop_number = this->config->perimeters + surface->extra_perimeters -1;  // 0-indexed loops
         
         Polygons gaps;
         
@@ -67,7 +66,7 @@ PerimeterGenerator::process()
             ThickPolylines thin_walls;
             
             // we loop one time more than needed in order to find gaps after the last perimeter was applied
-            for (signed short i = 0; i <= loop_number+1; ++i) {  // outer loop is 0
+            for (int i = 0; i <= loop_number+1; ++i) {  // outer loop is 0
                 Polygons offsets;
                 if (i == 0) {
                     // the minimum thickness of a single loop is:
@@ -170,16 +169,16 @@ PerimeterGenerator::process()
             }
             
             // nest loops: holes first
-            for (signed short d = 0; d <= loop_number; ++d) {
+            for (int d = 0; d <= loop_number; ++d) {
                 PerimeterGeneratorLoops &holes_d = holes[d];
                 
                 // loop through all holes having depth == d
-                for (signed short i = 0; i < holes_d.size(); ++i) {
+                for (int i = 0; i < (int)holes_d.size(); ++i) {
                     const PerimeterGeneratorLoop &loop = holes_d[i];
                     
                     // find the hole loop that contains this one, if any
-                    for (signed short t = d+1; t <= loop_number; ++t) {
-                        for (signed short j = 0; j < holes[t].size(); ++j) {
+                    for (int t = d+1; t <= loop_number; ++t) {
+                        for (int j = 0; j < (int)holes[t].size(); ++j) {
                             PerimeterGeneratorLoop &candidate_parent = holes[t][j];
                             if (candidate_parent.polygon.contains(loop.polygon.first_point())) {
                                 candidate_parent.children.push_back(loop);
@@ -191,8 +190,8 @@ PerimeterGenerator::process()
                     }
                     
                     // if no hole contains this hole, find the contour loop that contains it
-                    for (signed short t = loop_number; t >= 0; --t) {
-                        for (signed short j = 0; j < contours[t].size(); ++j) {
+                    for (int t = loop_number; t >= 0; --t) {
+                        for (int j = 0; j < (int)contours[t].size(); ++j) {
                             PerimeterGeneratorLoop &candidate_parent = contours[t][j];
                             if (candidate_parent.polygon.contains(loop.polygon.first_point())) {
                                 candidate_parent.children.push_back(loop);
@@ -207,16 +206,16 @@ PerimeterGenerator::process()
             }
         
             // nest contour loops
-            for (signed short d = loop_number; d >= 1; --d) {
+            for (int d = loop_number; d >= 1; --d) {
                 PerimeterGeneratorLoops &contours_d = contours[d];
                 
                 // loop through all contours having depth == d
-                for (signed short i = 0; i < contours_d.size(); ++i) {
+                for (int i = 0; i < (int)contours_d.size(); ++i) {
                     const PerimeterGeneratorLoop &loop = contours_d[i];
                 
                     // find the contour loop that contains it
-                    for (signed short t = d-1; t >= 0; --t) {
-                        for (signed short j = 0; j < contours[t].size(); ++j) {
+                    for (int t = d-1; t >= 0; --t) {
+                        for (int j = 0; j < contours[t].size(); ++j) {
                             PerimeterGeneratorLoop &candidate_parent = contours[t][j];
                             if (candidate_parent.polygon.contains(loop.polygon.first_point())) {
                                 candidate_parent.children.push_back(loop);
@@ -298,31 +297,22 @@ PerimeterGenerator::process()
             // two or more loops
             inset += pspacing/2;
         }
-        
         // only apply infill overlap if we actually have one perimeter
         if (inset > 0)
             inset -= this->config->get_abs_value("infill_overlap", inset + ispacing/2);
-        
-        {
-            ExPolygons expp = union_ex(last);
-            
-            // simplify infill contours according to resolution
-            Polygons pp;
-            for (ExPolygons::const_iterator ex = expp.begin(); ex != expp.end(); ++ex)
-                ex->simplify_p(SCALED_RESOLUTION, &pp);
-            
-            // collapse too narrow infill areas
-            coord_t min_perimeter_infill_spacing = ispacing * (1 - INSET_OVERLAP_TOLERANCE);
-            
-            // append infill areas to fill_surfaces
-            surfaces_append(
-                this->fill_surfaces->surfaces, 
-                offset2_ex(
-                    pp,
-                    -inset -min_perimeter_infill_spacing/2,
-                    +min_perimeter_infill_spacing/2),
-                stInternal);
-        }
+        // simplify infill contours according to resolution
+        Polygons pp;
+        for (ExPolygon &ex : union_ex(last))
+            ex.simplify_p(SCALED_RESOLUTION, &pp);
+        // collapse too narrow infill areas
+        coord_t min_perimeter_infill_spacing = ispacing * (1 - INSET_OVERLAP_TOLERANCE);
+        // append infill areas to fill_surfaces
+        this->fill_surfaces->append(
+            offset2_ex(
+                pp,
+                -inset -min_perimeter_infill_spacing/2,
+                +min_perimeter_infill_spacing/2),
+            stInternal);
     } // for each island
 }
 
@@ -354,41 +344,29 @@ PerimeterGenerator::_traverse_loops(const PerimeterGeneratorLoops &loops,
         if (this->config->overhangs && this->layer_id > 0
             && !(this->object_config->support_material && this->object_config->support_material_contact_distance.value == 0)) {
             // get non-overhang paths by intersecting this loop with the grown lower slices
-            {
-                Polylines polylines;
-                intersection((Polygons)loop->polygon, this->_lower_slices_p, &polylines);
-                
-                for (Polylines::const_iterator polyline = polylines.begin(); polyline != polylines.end(); ++polyline) {
-                    ExtrusionPath path(role);
-                    path.polyline   = *polyline;
-                    path.mm3_per_mm = is_external ? this->_ext_mm3_per_mm           : this->_mm3_per_mm;
-                    path.width      = is_external ? this->ext_perimeter_flow.width  : this->perimeter_flow.width;
-                    path.height     = this->layer_height;
-                    paths.push_back(path);
-                }
-            }
+            extrusion_paths_append(
+                paths,
+                intersection_pl(loop->polygon, this->_lower_slices_p),
+                role,
+                is_external ? this->_ext_mm3_per_mm           : this->_mm3_per_mm,
+                is_external ? this->ext_perimeter_flow.width  : this->perimeter_flow.width,
+                this->layer_height);
             
             // get overhang paths by checking what parts of this loop fall 
             // outside the grown lower slices (thus where the distance between
             // the loop centerline and original lower slices is >= half nozzle diameter
-            {
-                Polylines polylines;
-                diff((Polygons)loop->polygon, this->_lower_slices_p, &polylines);
-                
-                for (Polylines::const_iterator polyline = polylines.begin(); polyline != polylines.end(); ++polyline) {
-                    ExtrusionPath path(erOverhangPerimeter);
-                    path.polyline   = *polyline;
-                    path.mm3_per_mm = this->_mm3_per_mm_overhang;
-                    path.width      = this->overhang_flow.width;
-                    path.height     = this->overhang_flow.height;
-                    paths.push_back(path);
-                }
-            }
+            extrusion_paths_append(
+                paths,
+                diff_pl(loop->polygon, this->_lower_slices_p),
+                erOverhangPerimeter,
+                this->_mm3_per_mm_overhang,
+                this->overhang_flow.width,
+                this->overhang_flow.height);
             
             // reapply the nearest point search for starting point
             // We allow polyline reversal because Clipper may have randomly
             // reversed polylines during clipping.
-            paths = ExtrusionEntityCollection(paths).chained_path();
+            paths = (ExtrusionPaths)ExtrusionEntityCollection(paths).chained_path();
         } else {
             ExtrusionPath path(role);
             path.polyline   = loop->polygon.split_at_first_point();
@@ -413,7 +391,7 @@ PerimeterGenerator::_traverse_loops(const PerimeterGeneratorLoops &loops,
     // sort entities into a new collection using a nearest-neighbor search,
     // preserving the original indices which are useful for detecting thin walls
     ExtrusionEntityCollection sorted_coll;
-    coll.chained_path(&sorted_coll, false, &sorted_coll.orig_indices);
+    coll.chained_path(&sorted_coll, false, erMixed, &sorted_coll.orig_indices);
     
     // traverse children and build the final collection
     ExtrusionEntityCollection entities;
@@ -459,7 +437,7 @@ PerimeterGenerator::_variable_width(const ThickPolylines &polylines, ExtrusionRo
         ExtrusionPath path(role);
         ThickLines lines = p->thicklines();
         
-        for (size_t i = 0; i < lines.size(); ++i) {
+        for (int i = 0; i < (int)lines.size(); ++i) {
             const ThickLine& line = lines[i];
             
             const coordf_t line_len = line.length();
